#include "stdafx.h"
#include "GLFW/IncludesGLFW.h"
#include "FunctionsAndConstants.h"
#include "Frustum.h"

// extract a plane from a given matrix and row id
void DECLDIR ExtractPlane(plane_t &plane, GLfloat *mat, int row)
{
  int scale = (row < 0) ? -1 : 1;
  row = abs(row) - 1;

  // calculate plane coefficients from the matrix
  plane.A = mat[3] + scale * mat[row];
  plane.B = mat[7] + scale * mat[row + 4];
  plane.C = mat[11] + scale * mat[row + 8];
  plane.D = mat[15] + scale * mat[row + 12];

  // normalize the plane
  float length = sqrtf(plane.A * plane.A + plane.B * plane.B + plane.C * plane.C);
  plane.A /= length;
  plane.B /= length;
  plane.C /= length;
  plane.D /= length;
}

bool DECLDIR SphereInFrustum(Vector3D center, GLfloat radius, frustum_t frustum)
{
  GLfloat dist;
  for (int i = 0; i < 6; ++i)
  {
    dist = frustum.planes[i].A * center.x +
           frustum.planes[i].B * center.y +
           frustum.planes[i].C * center.z +
           frustum.planes[i].D;

    if (dist <= -radius)
      return false;
  }

  return true;
}


void DECLDIR ViewFrustum::Step()
{
	  // get the projection and modelview matrices
  GLfloat projection[16];
  GLfloat modelview[16];

  glGetFloatv(GL_PROJECTION_MATRIX, projection);
  glGetFloatv(GL_MODELVIEW_MATRIX, modelview);

  // use OpenGL to multiply them
  glPushMatrix();
  glLoadMatrixf(projection);
  glMultMatrixf(modelview);
  glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
  glPopMatrix();

  // extract each plane
  ExtractPlane(frustum.l, modelview, 1);
  ExtractPlane(frustum.r, modelview, -1);
  ExtractPlane(frustum.b, modelview, 2);
  ExtractPlane(frustum.t, modelview, -2);
  ExtractPlane(frustum.n, modelview, 3);
  ExtractPlane(frustum.f, modelview, -3);
}

bool DECLDIR ViewFrustum::FrustumCheck(Vector3D center, GLfloat radius)
{
	return SphereInFrustum(center, radius, frustum);
}