//Brent Cowan - Sept. 19 / 2010
#include "stdafx.h"
#include "GLFW/IncludesGLFW.h"

using namespace std ;

#include "FunctionsAndConstants.h"

const float DTR = 3.14159f/180.0f;
const float RTD = 57.295779f;      //(180/PI)


void DECLDIR DrawLine(Vector2D P1, Vector2D P2, float width)
{
	glLineWidth(width); 
	glBegin(GL_LINES); 
		glVertex3f(P1.x, P1.y, -4.0f); 
		glVertex3f(P2.x, P2.y, -4.0f); 
	glEnd();
}


void DECLDIR DrawCircle(Vector2D center, float radius, float width)
{
	Vector2D vec;

	glLineWidth(width); 
	glBegin(GL_LINE_LOOP); 

		for(int i = 0; i<24; i++)
		{
			vec = vec.AngleToVector(360.0f * (float(i)/24.0f));
			vec = vec*radius + center;
			glVertex3f(vec.x, vec.y, -4.0f); 
		}
		
	glEnd();
}

Vector2D DECLDIR ClosestPointSeg(Vector2D point1, Vector2D point2, Vector2D center)
{
	Vector2D vecV   = point2 - point1;
	Vector2D pointV = center - point1;
	Vector2D norm;
	float proj;

	//Prevent a division by zero error
	if(vecV.GetLength() < 0.00001f)return point1;

	//if point1 is the closest
	norm = vecV;
	norm.Normalize();
	proj = pointV.Dot(norm);
	if(proj <= 0.0f)return point1;

	//if point2 is closest
	if(proj >= vecV.GetLength())return point2;

	//The point is somewhere on the line
	return norm * proj + point1;
}
 
bool DECLDIR LineCircleCollision(Vector2D point1, Vector2D point2, Vector2D center, float radius)
{
	Vector2D closest = ClosestPointSeg(point1, point2, center);
	Vector2D dist_v = center - closest;

	if(dist_v.GetLength() > radius)return false;
	else return true;
}

float DECLDIR RotateToward(float currentDir, float desiredDir, float rotationSpeed)
{
	static int tempDir; //temporarly hold the direction faced
	static float difference;

	difference = currentDir-desiredDir;

	//Decide which direction to rotate
	tempDir = int(difference)%360;

	difference += float(int(difference/360.0f))*-360.0f;
	if(difference<0){difference+=360;}
	if(difference>180.0f){difference=360.0f-difference;}

	if(tempDir<0){tempDir+=360;}

	//if(difference<0.0f){difference*=-1.0f;}//Make the difference positive

	if(tempDir>180)//Must be turning left
	{
		if(difference>rotationSpeed)//Don't turn more that the rotationSpeed
		{
			currentDir += rotationSpeed;
		}
		else
		{
			currentDir += difference;
		}
	}
	else //Must be turning right
	{
		if(difference>rotationSpeed)//Don't turn more that the rotationSpeed
		{
			currentDir -= rotationSpeed;
		}
		else
		{
			currentDir -= difference;
		}
	}

	currentDir += float(int(currentDir/360.0f))*-360.0f;
	if(currentDir<0){currentDir+=360;}

	return currentDir;
}

bool DECLDIR CircleCircleCollision(Vector2D center1, float radius1, Vector2D center2, float radius2)
{
	static Vector2D vec;
	static float    dist;

	vec  = center1-center2;
	dist = radius1+radius2;
	dist = dist*dist;

	if(vec.GetLengthSq()<dist)
	{
		return true;
	}
	return false;
}

float DECLDIR DirectionDifference(float direction1, float direction2)
{
	static float difference;

	difference = DirectionInRange(direction2-direction1);
	if(difference>180.0f) difference = (360.0f-difference)*-1.0f;

	return difference;
}

float DECLDIR DirectionInRange(float direction)
{
	static float out;
	static int tempInt;

	out = direction;
	tempInt = int(out)%360;
	out = out -float(int(out))+float(tempInt);
	if(out<0.0f)out+=360.0f;
	
	return out;
}