
#pragma comment(lib, "Gadget/DevIL/DevIL.lib")
#pragma comment(lib, "Gadget/DevIL/ILU.lib")
#pragma comment(lib, "Gadget/Devil/ILUT.lib")

#include "DevIL/il.h"
#include "DevIL/ilu.h"
#include "DevIL/ilut.h"

#include "GLFW/IncludesGLFW.h"
#include "Extensions.h"

#include "Texture.h"
#include "Vector3D.h"

#include "Image.h"

bool Image::init = false;

Image::Image()
{
	width = 0; height = 0; GL_ID = 0; 
	clamp = false; 
	isNormal = false;
	keepData = false;
	data = NULL;
	erase = false;

	//Initialize DevIL if it hasn't been. 
	//Note: it is not a problem if this is called more than once
	if(!init)
	{
		ilInit(); iluInit();
		ilutRenderer(ILUT_OPENGL);
		init = true;
	}
}

Image::~Image()
{
	if(glIsTexture(GL_ID))
	{
		glDeleteTextures(1, &GL_ID);
	}
	
	if(data!=NULL){delete [] data; data = NULL;}
}

bool Image::Load(const char *FileName)
{
	if(data!=NULL){delete [] data; data = NULL;}

	//Delete the texture from OpenGL if it exists
	if(glIsTexture(GL_ID))
	{
		glDeleteTextures(1, &GL_ID);
	}

	//Load the image
	ILuint DevIL_ID;
    ilGenImages(1, &DevIL_ID);
    ilBindImage(DevIL_ID);

    if (!ilLoadImage(FileName)) 
	{
        ilDeleteImages(1, &DevIL_ID);
        return false;
    }

    //Convert paletted images to RGB format
    if (ilGetInteger(IL_IMAGE_FORMAT) == IL_COLOR_INDEX)
    {
        if (!ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE))
        {
            ilDeleteImages(1, &DevIL_ID);
            return false;
        }
    }

    //Generate mipmaps if necessary
    if (!iluBuildMipmaps())
    {
        ilDeleteImages(1, &DevIL_ID);
        return false;
    }

    //Bind the images to OpenGL
    glGenTextures(1, &GL_ID);
    glBindTexture(GL_TEXTURE_2D, GL_ID);

    //Get image information
    ILint  numMipmaps = ilGetInteger(IL_NUM_MIPMAPS);
    components = ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL);
    GLenum type       = (GLenum) ilGetInteger(IL_IMAGE_TYPE);
    GLenum format     = (GLenum) ilGetInteger(IL_IMAGE_FORMAT);
    width  = ilGetInteger(IL_IMAGE_WIDTH);
    height = ilGetInteger(IL_IMAGE_HEIGHT);
    
	//I added this!
	ilBindImage(DevIL_ID);
	ilActiveMipmap(0);
	GLsizei width  = ilGetInteger(IL_IMAGE_WIDTH);
    GLsizei height = ilGetInteger(IL_IMAGE_HEIGHT);

	data = new GLubyte[components*width*height];
	const GLubyte* imageData   = (GLubyte*) ilGetData();

	//for(int i=0; i<width*height*components; i++)
	//{
	//	data[i] = imageData[i];
	//	//data[i] = 20;
	//}
	GLubyte tempB;
	for(int w=0; w<width*components; w++){for(int h=0; h<height; h++)
	{
		tempB  = imageData[w+h*width*components];
		data[w+(height-h-1)*width*components] = tempB;
	}}//End triple for loop



	const GLvoid* curData   = (GLvoid*) data;
    glTexImage2D(GL_TEXTURE_2D,     // target
		0,          // mipmap level-of-detail
        components, // components (internal format)
        width,      // width
        height,     // height
        0,          // border
        format,     // format (GL_RGB, GL_RGBA...)
        type,       // type (GL_UNSIGNED_BYTE...)
        curData);       

	//Now create some mip maps!
	float tempf;
	GLsizei oldWidth  = width;
	GLsizei oldHeight = height;

	GLubyte* oldData = new GLubyte[components*width*height];
	for(int i=0; i<width*height*components; i++){oldData[i] = data[i];}

	//for(int i = 1; i<numMipmaps+1; i++)
	numMipmaps = 0;
	while(oldWidth>4 || oldHeight>4)
	{
		numMipmaps++;
		GLsizei newWidth  = oldWidth/2;
		GLsizei newHeight = oldHeight/2;
		GLubyte* newData = new GLubyte[components*newWidth*newHeight];
		for(int w=0; w<newWidth; w++){for(int h=0; h<newHeight; h++){for(int c=0; c<components; c++){
			tempf  = float(oldData[((w*2)  +(h*2)  *oldWidth)*components+c]);
			tempf += float(oldData[((w*2+1)+(h*2)  *oldWidth)*components+c]);
			tempf += float(oldData[((w*2)  +(h*2+1)*oldWidth)*components+c]);
			tempf += float(oldData[((w*2+1)+(h*2+1)*oldWidth)*components+c]);
			tempf += 0.5f;

			newData[c+w*components+h*newWidth*components] = GLubyte(tempf/4.0f);
		}}}//End triple for loop

		//If image is a normal map, normalize it
		if(isNormal)
		{
			Vector3D norm;
			for(int w=0; w<newWidth*components; w+=components){for(int h=0; h<newHeight; h++){
				norm.x  = float(newData[w   + (h*newWidth*components)]-128);
				norm.y  = float(newData[w+1 + (h*newWidth*components)]-128);
				norm.z  = float(newData[w+2 + (h*newWidth*components)]-128);
				
				norm.Normalize(); norm = norm*127.3f;
				norm.x+=128.0f; norm.y+=128.0f; norm.z+=128.0f;
				
				newData[w   + h*newWidth*components] = GLubyte(norm.x+0.5f);
				newData[w+1 + h*newWidth*components] = GLubyte(norm.y+0.5f);
				newData[w+2 + h*newWidth*components] = GLubyte(norm.z+0.5f);
			}}//End double for loop
		}

		{
		const GLvoid* curData   = (GLvoid*) newData;
		glTexImage2D(GL_TEXTURE_2D,     // target
			numMipmaps,          // mipmap level-of-detail
			components, // components (internal format)
			newWidth,   // width
			newHeight,  // height
			0,          // border
			format,     // format (GL_RGB, GL_RGBA...)
			type,       // type (GL_UNSIGNED_BYTE...)
			curData); 
		}
		oldWidth = oldWidth/2; oldHeight = oldHeight/2;
		delete [] oldData;
		oldData = new GLubyte[components*oldWidth*oldHeight];
		for(int i=0; i<oldWidth*oldHeight*components; i++){oldData[i] = newData[i];}
		
		delete [] newData; newData = NULL;
	}
	delete [] oldData; oldData = NULL;

    //Delete temporary IL image data
	ilDeleteImages(1, &DevIL_ID);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, numMipmaps);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	//Sometimes you need to clamp images to the border to hide the seam
	if(clamp)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	}
	
	/*int maxAnisotropy;
	glGetIntegerv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &maxAnisotropy);
	if(maxAnisotropy > int(G.window.textureQuality))
	{
		maxAnisotropy = int(G.window.textureQuality);
	}
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, maxAnisotropy);*/

	//Got this far, must have worked
	if(!keepData){delete [] data; data = NULL;}

    return true;
}

float Image::GetData(float U, float V, unsigned char component)
{
	if(data==NULL || component>components-1)return -1.0f;
	if(U<0.0f || U>1.0f)return -1.0f;
	if(V<0.0f || V>1.0f)return -1.0f;
	unsigned char out = data[(int(U*float(width)+0.5f)*components+component)+int((1.0f-V)*float(height))*width*components];
	return float(out)/255.0f;
}

void Image::Activate(){glBindTexture(GL_TEXTURE_2D, GL_ID);}

void Image::Clamp(){clamp = true;}

void Image::Step(){}
