//Brent Cowan May 22 2010
#include "stdafx.h"
#include "GLFW/IncludesGLFW.h"
#include "Vector2D.h"
#include "MouseManager.h"


DECLDIR MouseManager::MouseManager()
{
	leftButton = rightButton = leftPrevious = rightPrevious = false;
	middleButton = middlePrevious = false;
}

void DECLDIR MouseManager::Step()
{
	//Mouse movement
	glfwGetMousePos(&xPos, &yPos);
	position.x = float(xPos);
	position.y = float(yPos);
	glfwGetWindowSize(&xPos, &yPos);
	position.y = float(yPos)-position.y;

	smoothPosition = (smoothPosition+position)*0.5f;
	velocity = smoothPosition - lastPosition;
	lastPosition = smoothPosition;

	//Mouse Buttons
	leftPrevious    = leftButton;
	rightPrevious   = rightButton;
	middlePrevious  = middleButton;
	leftButton      = glfwGetMouseButton(GLFW_MOUSE_BUTTON_LEFT)>0;
	rightButton     = glfwGetMouseButton(GLFW_MOUSE_BUTTON_RIGHT)>0;
	middleButton    = glfwGetMouseButton(GLFW_MOUSE_BUTTON_MIDDLE)>0;
}

void DECLDIR MouseManager::SetVisibility(bool show)
{
	if(show)
	{
		glfwEnable(GLFW_MOUSE_CURSOR);
	}
	else
	{
		glfwDisable(GLFW_MOUSE_CURSOR);
	}
	ShowCursor(show);
}

Vector2D DECLDIR MouseManager::GetPosition()
{	
	return smoothPosition;
}

void DECLDIR MouseManager::SetPosition(Vector2D position)
{
	glfwGetWindowSize(&xPos, &yPos);
	this->position = position;
	this->position.y = float(yPos)-position.y;
	glfwSetMousePos(int(position.x+0.5f), int(this->position.y+0.5f));
	
	smoothPosition = this->position;
	smoothPosition.y = position.y;
	lastPosition   = smoothPosition;
}

void DECLDIR MouseManager::SetPosition(float x, float y)
{
	glfwGetWindowSize(&xPos, &yPos);
	this->position.x = x;
	this->position.y = float(yPos)-y;
	glfwSetMousePos(int(position.x+0.5f), int(this->position.y+0.5f));
	
	smoothPosition = this->position;
	smoothPosition.y = y;
	lastPosition   = smoothPosition;
}

Vector2D DECLDIR MouseManager::GetVelocity()
{
	return velocity;
}

bool DECLDIR MouseManager::GetLeftPress()
{
	return (leftButton && !leftPrevious);
}

bool DECLDIR MouseManager::GetRightPress()
{
	return (rightButton && !rightPrevious);
}

bool DECLDIR MouseManager::GetLeftHold()
{
	return leftButton;
}

bool DECLDIR MouseManager::GetRightHold()
{
	return rightButton;
}

bool DECLDIR MouseManager::GetLeftRelease()
{
	return (!leftButton && leftPrevious);
}

bool DECLDIR MouseManager::GetRightRelease()
{
	return (!rightButton && rightPrevious);
}

bool DECLDIR MouseManager::GetMiddleHold()
{
	return middleButton;
}

bool DECLDIR MouseManager::GetMiddleRelease()
{
	return (!middleButton && middlePrevious);
}

bool DECLDIR MouseManager::GetMiddlePress()
{
	return (middleButton && !middlePrevious);
}

float DECLDIR MouseManager::GetWheelPosition()
{
	return float(glfwGetMouseWheel());
}