//Brent Cowan May 22 2010
#include "stdafx.h"
#include "GLFW/IncludesGLFW.h"
#include "Vector2D.h"
#include "Vector3D.h"
#include "Frustum.h"
#include "WindowManager.h"
#include "TimeManager.h"
#include "KeyboardManager.h"
#include "MouseManager.h"
#include "JoystickManager.h"
#include "RenderingManager.h"

DECLDIR RenderingManager::RenderingManager()
{
	init        = false;
	window      = NULL;
	timer       = NULL;
	keyboard    = NULL;
	mouse       = NULL;
	joystick    = NULL;
	fieldOfView = 80.0f;
	nearClip    = 1.0f;
	farClip     = 10000.0f;
}

void DECLDIR RenderingManager::Init(WindowManager* window, TimeManager* timer, KeyboardManager* keyboard, MouseManager* mouse, JoystickManager* joystick)
{
	this->window    = window;
	this->timer     = timer;
	this->keyboard  = keyboard;
	this->mouse     = mouse;
	this->joystick  = joystick;
	init            = true;
}

void DECLDIR RenderingManager::BeginFrame()
{
	if(init)
	{
		if(window->GetClearColorBuffer())
		{
			clearColor = window->GetClearColor();
			glClearColor(clearColor.R, clearColor.G, clearColor.B, clearColor.A);
			if(window->GetClearDepthBuffer())
			{
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			}
			else
			{
				glClear(GL_COLOR_BUFFER_BIT);
			}
		}
		else
		{
			if(window->GetClearDepthBuffer())
			{
				glClear(GL_DEPTH_BUFFER_BIT);
			}
		}
	}

	timer->Step();
	keyboard->Step();
	mouse->Step();
	joystick->Step();
}

void DECLDIR RenderingManager::EndFrame()
{
	glfwSwapBuffers();
}

void DECLDIR RenderingManager::Render2D()
{
	//render in ortho for 2D elements such as text & sprites
	glMatrixMode(GL_PROJECTION);			// set projection matrix current matrix
	glLoadIdentity();						// reset projection matrix
	glOrtho(0.0f, window->GetWidth(), 0.0f, window->GetHeight(), nearClip, farClip);
	glMatrixMode(GL_MODELVIEW);				// set modelview matrix
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -1.0);
}

void DECLDIR RenderingManager::Render3D()
{
	// renders in perspective
	// load the identity matrix (clear to default position and orientation)
	glMatrixMode(GL_PROJECTION);			// set projection matrix current matrix
	glLoadIdentity();						// reset projection matrix
	gluPerspective(fieldOfView, window->GetAspectRatio(), nearClip, farClip);
	glMatrixMode(GL_MODELVIEW);				// set modelview matrix
	glLoadIdentity();
}

void DECLDIR RenderingManager::SetFieldOfView(float fieldOfView)
{
	if(fieldOfView>0.0f && fieldOfView<180.0f)
	{
		this->fieldOfView = fieldOfView;
	}
}

void DECLDIR RenderingManager::SetNearClippingPlane(float nearClippingPlane)
{
	this->nearClip = nearClippingPlane;
}

void DECLDIR RenderingManager::SetFarClippingPlane(float farClippingPlane)
{
	this->farClip = farClippingPlane;
}