//Brent Cowan - May. 19 / 2008

#include <fstream>         //Loading and saving files
using namespace std;

#include "GLFW/IncludesGLFW.h"
#include "Extensions.h"
#include "ShaderClass.h"

ShaderClass::ShaderClass()
{
	textures = 0;
	shaderProgram = NULL;
	f[0]=0.0f; f[1]=0.0f;  f[2]=0.0f;  f[3]=0.0f;  f[4]=0.0f; 
}

ShaderClass::~ShaderClass()
{
	ext::glUseProgramObjectARB(0);
	ext::glDetachObjectARB(shaderProgram, glVertexShader);
	ext::glDetachObjectARB(shaderProgram, glPixelShader);
	ext::glDeleteObjectARB(shaderProgram);
}

bool ShaderClass::LoadFromFile(char *vertexShader, char *pixelShader, bool debug)
{
	int result;
	shaderProgram = ext::glCreateProgramObjectARB();

	//Load the vertex shader.
	char *ShaderCode = LoadShader(vertexShader);
	glVertexShader = ext::glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	ext::glShaderSourceARB(glVertexShader, 1, (const char**)&ShaderCode, NULL);
	ext::glCompileShaderARB(glVertexShader);
	ext::glGetObjectParameterivARB(glVertexShader, GL_OBJECT_COMPILE_STATUS_ARB, &result);
	ext::glAttachObjectARB(shaderProgram, glVertexShader);

	delete[] ShaderCode;
	ShaderCode = NULL;

	//Load the pixel shader.
	ShaderCode = LoadShader(pixelShader);
	glPixelShader = ext::glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	ext::glShaderSourceARB(glPixelShader, 1, (const char**)&ShaderCode, NULL);
	ext::glCompileShaderARB(glPixelShader);
	ext::glGetObjectParameterivARB(glPixelShader, GL_OBJECT_COMPILE_STATUS_ARB, &result);
	ext::glAttachObjectARB(shaderProgram, glPixelShader);

	delete[] ShaderCode;
	ShaderCode = NULL;

	// Link our shaders.
	ext::glLinkProgramARB(shaderProgram);
	ext::glGetObjectParameterivARB(shaderProgram, GL_OBJECT_LINK_STATUS_ARB, &result);

	if(debug)
	{
		char buff[9000];
		GLsizei len =9000;
		GLsizei outlen;
		//glGetInfoLogARB(shaderProgram, len, &outlen, buff);
		

		//Write the file to disk
		ofstream out;
		//create the file
		out.open("ShaderReport.html", ios::out | ios::binary);

		if(!out.fail()) //if we can write to the file
		{
			//html
			out << "<HTML><HEAD><TITLE>Shader Report</TITLE><BODY> ";

			//Output vertex shader debug
			out << "<H2>Vertex Shader File: " << vertexShader << "</H2>";
			ext::glGetInfoLogARB(glVertexShader, len, &outlen, buff);
			if(outlen != NULL)
			{
				out << "<FONT color=#ff0000>";
				for(int i=0; i<outlen; i++)
				{
					if(buff[i] == '\n')
					{
						out << "<BR>";
					}
					else
					out << buff[i];
				}
				out << "</FONT>";
			}
			else{out << endl << "(Compiled Successfully)" << "<BR>";}
			out << "<BR>";

			//Output pixel shader debug
			out << "<H2>Pixel Shader File:  " << pixelShader << "</H2>";
			ext::glGetInfoLogARB(glPixelShader, len, &outlen, buff);
			if(outlen != NULL)
			{
				out << "<FONT color=#ff0000>";
				for(int i=0; i<outlen; i++)
				{
					if(buff[i] == '\n')
					{
						out << "<BR>";
					}
					else
					out << buff[i];
				}
				out << "</FONT>";
			}
			else{out << "(Compiled Successfully)" << "<BR>";}
			out << "<BR>";

			//Output Link status
			out << "<H2>Linking Status:  " << "</H2>";
			ext::glGetInfoLogARB(shaderProgram, len, &outlen, buff);
			if(outlen != NULL)
			{
				for(int i=0; i<outlen; i++)
				{
					if(buff[i] == '\n')
					{
						out << "<BR>";
					}
					else
					out << buff[i];
				}
				out << "<BR>";
			}

			//ending html
			out << "</BODY></HTML> ";

			out.close();
		}
	}

	if(!result)return false;

	// Bind our shader textures
	if(textures>0){tex[0] = ext::glGetUniformLocationARB(shaderProgram, "tex0");}
	if(textures>1){tex[1] = ext::glGetUniformLocationARB(shaderProgram, "tex1");}
	if(textures>2){tex[2] = ext::glGetUniformLocationARB(shaderProgram, "tex2");}

	v[0] = ext::glGetUniformLocationARB(shaderProgram, "v0");
	v[1] = ext::glGetUniformLocationARB(shaderProgram, "v1");
	v[2] = ext::glGetUniformLocationARB(shaderProgram, "v2");
	v[3] = ext::glGetUniformLocationARB(shaderProgram, "v3");
	v[4] = ext::glGetUniformLocationARB(shaderProgram, "v4");

	return true;
}

char* ShaderClass::LoadShader(char *shaderName)
{
   char tempLine[256] = {0};
   int count = 0;

   // input object.
   ifstream tempInput;

   // Open the shader file.
   tempInput.open(shaderName);

   // If there is any errors then return false.
   if(!tempInput.is_open())
      {
         return NULL;
      }


   // Loop through each line of the file and get the total size.
   while(!tempInput.eof())
      {
         tempInput.getline(tempLine, 256, '\n');
         count++;
      }

   // Close the shader.
   tempInput.close();

   if(count == 0) return NULL;

   // Read in the data for use this time.
   ifstream input;

   // Create array to hold shader code.
   char *ShaderSource = new char[256 * count];

   // Re-open the shader and read in the whole thing into the array.
   input.open(shaderName);
   input.getline(ShaderSource , 256 * count, '\0');

   input.close(); // Close the shader.

   return ShaderSource;
}

void ShaderClass::Activate()
{
	ext::glUseProgramObjectARB(shaderProgram);

	// Send the texture unit our shader texture will use.
	if(textures>0)
	{
		ext::glUniform1iARB(tex[0], 0);

		if(textures>1)
		{
			ext::glUniform1iARB(tex[1], 1);

			if(textures>2){ext::glUniform1iARB(tex[2], 2);}
		}
	}

	if(v[0]!=0)ext::glUniform4fARB(v[0], vec[0].x, vec[0].y, vec[0].z, f[0]);
	if(v[1]!=0)ext::glUniform4fARB(v[1], vec[1].x, vec[1].y, vec[1].z, f[1]);
	if(v[2]!=0)ext::glUniform4fARB(v[2], vec[2].x, vec[2].y, vec[2].z, f[2]);
	if(v[3]!=0)ext::glUniform4fARB(v[3], vec[3].x, vec[3].y, vec[3].z, f[3]);
	if(v[4]!=0)ext::glUniform4fARB(v[4], vec[4].x, vec[4].y, vec[4].z, f[4]);


}

void ShaderClass::Deactivate()
{
	ext::glUseProgramObjectARB(0);
}

void ShaderClass::Update()
{
	if(v[0]!=0)ext::glUniform4fARB(v[0], vec[0].x, vec[0].y, vec[0].z, f[0]);
	if(v[1]!=0)ext::glUniform4fARB(v[1], vec[1].x, vec[1].y, vec[1].z, f[1]);
	if(v[2]!=0)ext::glUniform4fARB(v[2], vec[2].x, vec[2].y, vec[2].z, f[2]);
	if(v[3]!=0)ext::glUniform4fARB(v[3], vec[3].x, vec[3].y, vec[3].z, f[3]);
	if(v[4]!=0)ext::glUniform4fARB(v[4], vec[4].x, vec[4].y, vec[4].z, f[4]);
}
