#include "EngineCore.h"
#include "TextBox.h"

Core* TextBox::G;

TextBox::TextBox()
{
	init       = false;
	G          = NULL;
	scale      = 24.0f;
	editable   = false;
	erase      = false;
	center     = false;
	flagChange = true;
	maxLength  = 65535;
}

TextBox::~TextBox(){}

bool TextBox::Init(Core* G, Font* font, float width, float height)
{
	if(G && font)
	{
		TextBox::G     = G;
		this->font     = font;
		font->fontSize = scale;
		init           = true;
		flagChange     = true;
		this->width    = width;
		this->height   = height;
		flagChange     = true;

		return true;
	}

	return false;
}

void TextBox::Step()
{
	//Use keyboard input
	if(editable)
	{
		if(G->Keyboard.KeyPress(GLFW_KEY_BACKSPACE))
		{
			text = text.substr(0, text.size()-1);
			flagChange = true;
		}

		//Letters
		for(unsigned char i=32; i<127; i++)
		{
			if(G->Keyboard.KeyPress(i) && text.size()<maxLength)
			{
				text += char(G->Keyboard.CorrectCase(i));

				flagChange  = true;
			}
		}
	}

	Render();
}

void TextBox::Render()
{
	if(init)//if the font has been loaded
	{
		while(flagChange)
		{
			CalculateWordWrap();
		}

		//Render the text
		for(unsigned short i=0; i<rows.size(); i++)
		{
			if(center)
			{
				font->Render(position.x + (width-rows[i].width)*0.5f, (position.y-scale*0.7f)-scale*i*0.8f, rows[i].text);
			}
			else
			{
				font->Render(position.x, (position.y-scale*0.7f)-scale*i*0.8f, rows[i].text);
			}
		}
	}
}

void TextBox::CalculateWordWrap()
{
	font->fontSize = scale;

	if(flagChange)
	{
		Row tempRow; tempRow.firstChar = 0;
		rows.clear();
		unsigned short tempString = 0;
		unsigned short lineChars = 0;

		//Calculate word wrap
		rows.push_back(tempRow);
		float current = 0.0f;
		for(unsigned short i=0; i<text.size(); i++)
		{
			current += font->GetCharWidth(text.c_str()[i]);
			if(text.c_str()[i]==char(10))//if we have found a new line character
			{	
				rows[rows.size()-1].text = rows[rows.size()-1].text.substr(0, rows[rows.size()-1].text.size());
				
				tempRow.firstChar = i;
				rows.push_back(tempRow);
				current = 0.0f;
				tempString = lineChars = 0;
			}
			else if(current>=width)//if we have reached the end
			{	
				if(tempString == lineChars)//if there are no spaces
				{
					rows[rows.size()-1].text = rows[rows.size()-1].text.substr(0, rows[rows.size()-1].text.size());
					i -= 1;
				}
				else//There are spaces, wrap at last space
				{
					rows[rows.size()-1].text = rows[rows.size()-1].text.substr(0, rows[rows.size()-1].text.size()-tempString);
					i -= tempString+1;
				}
				
				tempRow.firstChar = i;
				rows.push_back(tempRow);
				current = 0.0f;
				tempString = lineChars = 0;
			}
			else
			{
				rows[rows.size()-1].text += text.c_str()[i];
				tempString++; lineChars++;
			
				if(text.c_str()[i] == ' '){tempString = 0;}
			}
		}

		if(center)
		{
			for(unsigned short i=0; i<rows.size(); i++)
			{
				rows[i].width = font->GetLength(rows[i].text);
			}
		}

		flagChange = false;
	}

	if(scale*rows.size()*0.8f>height)
	{
		text = text.substr(rows[1].firstChar+1, text.size()-(rows[1].firstChar+1));
		flagChange = true;
	}
}

void TextBox::AddText(string text)
{
	this->text += text;
	if(this->text.size()>maxLength)
	{
		this->text = this->text.substr(0, maxLength);
	}
	flagChange = true;
}

void TextBox::AddNewLine()
{
	this->text += char(10);
	if(this->text.size()>maxLength)
	{
		this->text = this->text.substr(0, maxLength);
	}
	flagChange = true;
}

float TextBox::GetHeight()
{
	return height;
}

float TextBox::GetScale()
{
	return scale;
}

string TextBox::GetText()
{
	return text;
}

float TextBox::GetWidth()
{
	return width;
}

void TextBox::SetHeight(float height)
{
	this->height = height;
	flagChange  = true;
}

void TextBox::SetScale(float scale)
{
	this->scale = scale;
	flagChange  = true;
}

void TextBox::SetText(string text)
{
	this->text = text;
	flagChange = true;
}

void TextBox::SetText(char* text)
{
	this->text = text;
	if(this->text.size()>maxLength)
	{
		this->text = this->text.substr(0, maxLength);
	}
	flagChange = true;
}

void TextBox::SetWidth(float width)
{
	this->width = width;
	flagChange  = true;
}

bool TextBox::PointOnText(Vector2D point)
{
	if(point.x < position.x || point.x > position.x+width)return false;

	if(point.y < position.y-rows.size()*scale*0.8f-scale*0.1f || point.y > position.y)return false;

	return true;
}

void TextBox::AlignCenter()
{
	if(!center)flagChange  = true;
	center = true;
}

void TextBox::AlignRight()
{
	if(center)flagChange  = true;
	center = false;
}