//Brent Cowan May 22 2010
#include "stdafx.h"

#include "GLFW/IncludesGLFW.h"
#include "Mmsystem.h"
#include "Extensions.h"
#include "FunctionsAndConstants.h"
#include "WindowManager.h"

DECLDIR WindowManager::WindowManager()
{	
	width = 800; height = 600;
	x = 100; y = 50;
	depthBufferBits = 16;
	fullScreen = false;
	clearColorBuff = true;
	clearDepthBuff = true;
	clearColor.R = 0.0; clearColor.G = 0.0; clearColor.B = 0.0; clearColor.A = 1.0;
	multisample = true;
	samples = 2;
	textureQuality = 2;
	aspectRatio = float(width)/float(height);
	title = "Gadget3D";
	hDC = NULL;
	hWnd = NULL;
}

bool DECLDIR WindowManager::Open()
{
	bool success;

	if(multisample){glfwOpenWindowHint(GLFW_FSAA_SAMPLES, samples);}
	glfwOpenWindowHint(GLFW_WINDOW_NO_RESIZE, GL_TRUE);

	if(fullScreen)
	{
		success = glfwOpenWindow(width, height, 8, 8, 8, 0, depthBufferBits, 0, GLFW_FULLSCREEN)>0;
	}
	else
	{
		success = glfwOpenWindow(width, height, 8, 8, 8, 0, depthBufferBits, 0, GLFW_WINDOW)>0;
	}

	if(success)
	{
		srand(timeGetTime());
		string tempString = ToString(rand());
		glfwSetWindowTitle(tempString.c_str());
		hWnd = FindWindow(NULL, tempString.c_str());
		hDC = GetDC(hWnd);
		glfwSetWindowTitle(title.c_str());
		aspectRatio = float(width)/float(height);
		glfwSetWindowPos(x, y);
		
		glfwEnable(GLFW_MOUSE_CURSOR);

		glEnable(GL_DEPTH_TEST);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glEnable(GL_COLOR_MATERIAL);
		glEnable(GL_CULL_FACE);  // face culling on
		glCullFace(GL_BACK);    // front culling

		InitExtensions();

		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glEnableClientState(GL_SECONDARY_COLOR_ARRAY);
	}

	return success;
}

void DECLDIR WindowManager::Close()
{
	glfwCloseWindow();
}

void DECLDIR WindowManager::SetSize(int width, int height)
{
	this->height = height;
	this->width = width;
	glfwSetWindowSize(width, height);
	aspectRatio = float(width)/float(height);
}

void DECLDIR WindowManager::SetPosition(int x, int y)
{
	this->x = x;
	this->y = y;
	glfwSetWindowPos(x, y);
}

void DECLDIR WindowManager::SetTitle(const char* title)
{
	this->title = title;
	glfwSetWindowTitle(this->title.c_str());
}

bool DECLDIR WindowManager::IsOpen()
{
	return glfwGetWindowParam(GLFW_OPENED)>0;
}

void DECLDIR WindowManager::SetDepthBufferBits(int bits)
{
	if(bits<20)
	{
		depthBufferBits = 16;
	}
	else if(bits<28)
	{
		depthBufferBits = 24;
	}
	else
	{
		depthBufferBits = 32;
	}
}

void DECLDIR WindowManager::SetFullScreen(bool fullScreen)
{
	this->fullScreen = fullScreen;
}

void DECLDIR WindowManager::SetClearColor(float r, float g, float b, float a)
{
	if(r<=1.0f && r>=0.0f){clearColor.R = r;}
	if(g<=1.0f && g>=0.0f){clearColor.G = g;}
	if(b<=1.0f && b>=0.0f){clearColor.B = b;}
	if(a<=1.0f && a>=0.0f){clearColor.A = a;}
}

void DECLDIR WindowManager::SetClearColor(float r, float g, float b)
{
	if(r<=1.0f && r>=0.0f){clearColor.R = r;}
	if(g<=1.0f && g>=0.0f){clearColor.G = g;}
	if(b<=1.0f && b>=0.0f){clearColor.B = b;}
}

void DECLDIR WindowManager::SetClearColorBuffer(bool clear)
{
	clearColorBuff = clear;
}

void DECLDIR WindowManager::SetClearDepthBuffer(bool clear)
{
	clearDepthBuff = clear;
}

void DECLDIR WindowManager::SetSampleRate(int rate)
{
	if(rate<=1)
	{
		multisample = false;
		samples = 1;
	}
	else if(rate<=3)
	{
		multisample = true;
		samples = 2;
	}
	else if(rate<=6)
	{
		multisample = true;
		samples = 4;
	}
	else if(rate<=13)
	{
		multisample = true;
		samples = 8;
	}
	else if(rate<=128)
	{
		multisample = true;
		samples = 16;
	}
	else
	{
		multisample = true;
		samples = 2;
	}
}

void DECLDIR WindowManager::SetTextureQuality(int Quality)
{
	textureQuality = Quality;
}

void DECLDIR WindowManager::SetVerticalSync(bool vSync)
{
	if(ext::wglSwapIntervalEXT)
    {
		if(vSync)
		{
			ext::wglSwapIntervalEXT(1);
		}
		else
		{
			ext::wglSwapIntervalEXT(0);
		}
    }
}

bool DECLDIR WindowManager::GetClearColorBuffer()
{
	return clearColorBuff;
}

bool DECLDIR WindowManager::GetClearDepthBuffer()
{
	return clearDepthBuff;
}

Color4f DECLDIR WindowManager::GetClearColor()
{
	return clearColor;
}

int DECLDIR WindowManager::GetHeight()
{
	return height;
}

int DECLDIR WindowManager::GetWidth()
{
	return width;
}

HDC DECLDIR WindowManager::GetHDC()
{
	return hDC;
}

float DECLDIR WindowManager::GetAspectRatio()
{
	return aspectRatio;
}